use std::io::stdin;
use anyhow::Result;

fn main() -> Result<()> {
    let mut sums = (0, 0);

    for line in stdin().lines() {
        let line = line?;
        let sequence: Vec<isize> = line.split_whitespace()
            .map(str::parse)
            .collect::<Result<_,_>>()?;

        let preds = predict(&sequence);
        sums.0 += preds.0;
        sums.1 += preds.1;
    }
        
    println!("Sums are {sums:?}");

    Ok(())
}

fn predict(s: &[isize]) -> (isize,isize) {
    if s.iter().all(|&x| x == 0) {
        (0,0)
    } else {
        let diff = differentiate(s);
        let (a,b) = predict(&diff);
        (s[0] - a,  s[s.len()-1] + b)
    }
}




fn differentiate(s: &[isize]) -> Vec<isize> {
    s.iter().skip(1).zip(s).map(|(b,a)| b-a).collect()
}
